(***********************************************************************)
(*                                                                     *)
(*                             Pio (OCaml)                             *)
(*                                                                     *)
(*                    Polyglot Input/Output Library                    *)
(*                                                                     *)
(*                  projet Estime, INRIA-Rocquencourt                  *)
(*                                                                     *)
(*  Copyright 2004-2011 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: my_command_option.ml,v 1.2 2011-02-23 17:43:29 fclement Exp $ *)

open Ucommand_option;;

(* Documentation shortends. *)

let doc_with_valid_list = Printf.sprintf "%s\n\tValid entries are %s.";;

let doc_with_default = Printf.sprintf "%s\n\tDefault is %s.";;

let doc_with_valid_list_and_default s1 s2 s3 =
  doc_with_default (doc_with_valid_list s1 s2) s3;;

(* Report, debug and errors. *)

let make_flag default =
  let r = ref default in
  (fun () -> !r),
  (fun () -> r := not default);;

let make_antonymic_flags b =
  let get, set = make_flag b in
  let get_antonymic, set_antonymic = make_flag (not b) in
  let set () = set (); set_antonymic () in
  let set_antonymic () = set_antonymic (); set () in
  get, set, get_antonymic, set_antonymic;;

let mk_embedded_flags () =
  let get_report, set_report = make_flag false in
  let get_debug, set_debug = make_flag false in
  let set_debug () = set_debug (); set_report () in
  get_report, set_report, get_debug, set_debug;;

let mk_opt kind name set =
  let key = String.lowercase name in
  let doc = if kind = "debug" then "\n\t(implies report mode)" else "" in
  {key = Printf.sprintf "--%s_%s" kind key;
   spec = Arg.Unit set;
   doc =
     Printf.sprintf
       "  Turn on the %s library %s mode%s. Default is off."
     name kind doc};;

let report_opt = mk_opt "report"
and debug_opt = mk_opt "debug";;

let mk_short_key kind name =
  let key = String.lowercase name in
  Printf.sprintf "-%c%s" kind.[0] key;;

let report_short_key = mk_short_key "report"
and debug_short_key = mk_short_key "debug";;

let mk_fatal_error name =
  let msg = Printf.sprintf "Fatal error in %s library, please report:" name in
  (fun s ->
    prerr_endline (Printf.sprintf "%s\n %s" msg s);
    exit 2);;

let add_debug_flag long_name short_name =
  let get_debug, set_debug = make_flag false in
  let d_opt =
    debug_opt long_name set_debug,
    debug_short_key short_name in
  Ucommand_option.add_debug_bi_all [d_opt];
  let fatal_error = mk_fatal_error long_name in
  get_debug, fatal_error;;

let add_user_debug_flag long_name short_name =
  let get_debug, set_debug = make_flag false in
  let d_user_opt =
    (debug_opt long_name set_debug,
     Arg.Unit ignore),
    debug_short_key short_name in
  Ucommand_option.add_user_debug_bi_all [d_user_opt];
  let fatal_error = mk_fatal_error long_name in
  get_debug, fatal_error;;

let add_report_and_debug_flags long_name short_name =
  let get_report, set_report, get_debug, set_debug = mk_embedded_flags () in
  let r_opt =
    report_opt long_name set_report,
    report_short_key short_name in
  Ucommand_option.add_report_bi_all [r_opt]; 
  let d_opt =
    debug_opt long_name set_debug,
    debug_short_key short_name in
  Ucommand_option.add_debug_bi_all [d_opt];
  let fatal_error = mk_fatal_error long_name in
  get_report, get_debug, fatal_error;;

let add_user_report_and_debug_flags long_name short_name =
  let get_report, set_report, get_debug, set_debug = mk_embedded_flags () in
  let r_u_opt =
    (report_opt long_name set_report,
     Arg.Unit ignore),
    report_short_key short_name in
  Ucommand_option.add_user_report_bi_all [r_u_opt];
  let d_user_opt =
    (debug_opt long_name set_debug,
     Arg.Unit ignore),
    debug_short_key short_name in
  Ucommand_option.add_user_debug_bi_all [d_user_opt];
  let fatal_error = mk_fatal_error long_name in
  get_report, get_debug, fatal_error;;
