(***********************************************************************)
(*                                                                     *)
(*                          Sklml                                      *)
(*                                                                     *)
(*             Pierre Weis (Pierre.Weis@inria.fr)                      *)
(*             Francois Clement (Francois.Clement@inria.fr)            *)
(*             Quentin Carbonneaux (Quentin.Carbonneaux@inria.fr)      *)
(*                                                                     *)
(* This program is free software.                                      *)
(* (see the file LICENSE in the root directory of the distribution).   *)
(*                                                                     *)
(***********************************************************************)

(* $Id: stream_on_wire.mli,v 1.16 2010-11-22 10:23:52 qcarbonn Exp $ *)

(** This is the core of the stream feature for Sklml. *)

(** Streams:
 No dynamically built thunk can be passed on the wires (only plain values are
 transmitted).
 Hence, there is no lazy value in the streams for wires.
 There is still some kind of lazy evaluation in the computational networks,
 since values are obtained on demand.
 Hence, the modelization uses lazy evaluation to modelize on demand access to
 the values in streams. *)

(** {6 Wire Data} *)

type 'a wire_data =(* private *)
   | Data of 'a
   | Begin_sub_stream
   | End_sub_stream
;;
(** The type that specifies what values are communicated on the wires,
    either real wires in the parallel interpretation, or on simulated wires
    in the sequential interpretation. *)

(** {6 Streams} *)
type 'a cell = private
   | Nil
   | Cons of 'a wire_data * 'a stream

and 'a stream = 'a cell Lazy.t
;;
(** The type of streams that run on the wires in parallel or
  are infinite data structures of lazy streams with ['a wire_data] elements.
*)

(** Construction functions for cells. *)

val nil_cell : 'a cell;;
val cons_wire_data_cell : 'a wire_data * 'a stream -> 'a cell;;
val cons_data_cell : 'a * 'a stream -> 'a cell;;
val cons_begin_cell : 'a stream -> 'a cell;;
val cons_end_cell : 'a stream -> 'a cell;;

(** Access functions for cells *)
val get_cell : 'a stream -> 'a cell;;
(** [get_cell s] forces the stream [s] and returns its first cell. *)

(** Construction functions for streams. *)

val nil : 'a stream;;
val cons : 'a * 'a stream -> 'a stream;;

val cons_sub_stream : 'a stream * 'a stream -> 'a cell;;
(** [cons_sub_stream sub s] returns a new stream with elements of stream
  [sub] as a sub stream at the beginning of [s].
  Neither [sub] nor [s] are forced in this operation. *)

val append : 'a stream -> 'a stream -> 'a stream;;
