CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C                            Pio (Fortran)                            C
C                                                                     C
C                    Polyglot Input/Output Library                    C
C                                                                     C
C                  projet Estime, INRIA-Rocquencourt                  C
C                                                                     C
C  Copyright 2004-2011 INRIA.                                         C
C  Institut National de Recherche en Informatique et en Automatique.  C
C  All rights reserved. This file is distributed only by permission.  C
C                                                                     C
C  Francois Clement <Francois.Clement@inria.fr>                       C
C  Pierre Weis <Pierre.Weis@inria.fr>                                 C
C  Hend Ben Ameur <hbenameur@yahoo.ca>                                C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C $Id: buffers.F,v 1.8 2011-03-24 16:16:00 fclement Exp $

C     
C     Routines to handle scanning (io = PIO_IN) and printing
C     (io = PIO_OUT) buffers.
C     
C     Note: dealing with buffers, all the subroutines here should
C     include a include 'buffers_definition.fh' statement, or being
C     called only by such subroutines.
C
      
#include "pio_constants.h"
      
      subroutine invalidate_buffer(io)
C     Invalidate io buffer.
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input argument.
      integer io
      
      buffer_ptr(io) = INVALID_POINTER
      buffer_lim(io) = INVALID_LIMIT
      
      return
      end
      
      
      logical function end_of_buffer(io)
C     Is the buffer pointer beyond the limit?
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input argument.
      integer io
      
      end_of_buffer = buffer_ptr(io) .gt. buffer_lim(io)
      
      return
      end
      
      
      subroutine check_buffer_invariant(sbr, io)
C     Check the invariant on buffer pointers.
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io
      
C     Local variables.
      logical out_of_bounds, invalid_data_access, overflow
      integer bp, bl
      
C     External function.
      logical end_of_buffer
      
      bp = buffer_ptr(io)
      bl = buffer_lim(io)
      
      out_of_bounds = bp .lt. MIN_BUFFER_LOCATION
      invalid_data_access = bp .gt. bl
      overflow = end_of_buffer(io)
      
      if (out_of_bounds) then
         call error_out_of_bounds(sbr, io, 'buffer pointer')
      endif
      
      if (invalid_data_access) then
         call error_invalid_data_access
     +        (sbr, io, 'buffer', 'buffer pointer', bl, bp)
      endif
      
      if (overflow) then
         call error_overflow(sbr, io, 'buffer')
      endif
      
      return
      end
      
      
      subroutine init_buffer(sbr, io)
C     Initialize io buffer.
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io
      
      call invalidate_buffer(io)
      
      buffer_lim(io) = MIN_BUFFER_LOCATION
      buffer_ptr(io) = MIN_BUFFER_LOCATION
      
      call check_buffer_invariant(sbr, io)
      
      return
      end
      
      
      subroutine set_buffer_lim(sbr, io, lim)
C     Set io buffer limit to lim.
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io, lim
      
      buffer_lim(io) = lim
      
      call check_buffer_invariant(sbr, io)
      
      return
      end
      
      
      subroutine advance_buffer_lim(sbr, io, len)
C     Advance the io buffer limit of len locations.
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io, len
      
      call set_buffer_lim(sbr, io, buffer_lim(io) + len)
      
      return
      end
      
      
      subroutine incr_buffer_lim(sbr, io)
C     Increment the io buffer limit.
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io
      
      call advance_buffer_lim(sbr, io, 1)
      
      return
      end
      
      
      subroutine advance_buffer_ptr(sbr, io, len)
C     Advance the io buffer pointer of len locations.
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io, len
      
      buffer_ptr(io) = buffer_ptr(io) + len
      
      call check_buffer_invariant(sbr, io)
      
      return
      end
      
      
      subroutine incr_buffer_ptr(sbr, io)
C     Increment the pointer buffer_ptr(io).
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io
      
      call advance_buffer_ptr(sbr, io, 1)
      
      return
      end
      
      
      integer function get_buffer_ptr(io)
C     Get io buffer pointer.
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input argument.
      integer io
      
      get_buffer_ptr = buffer_ptr(io)
      
      return
      end
      
      
      subroutine set_buffer_char(sbr, io, i, c)
C     Set the ith character of the io buffer to c.
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input arguments.
      character sbr*(*), c
      integer io, i
      
      if (i .gt. MAX_BUFFER_LOCATION) then
         call error_buffer_overflow(sbr, io)
      endif
      
      buffer(io)(i:i) = c
      
      return
      end
      
      
      subroutine set_buffer_string(sbr, io, i, len, s)
C     Set the characters i to i+len-1 of the io buffer to s.
      
      implicit none
      
      include 'buffer_name_definition.fh'
      
C     Input arguments.
      character sbr*(*), s*(*)
      integer io, i, len
      
C     Local variables.
      integer j, k
      
C     External function.
      logical pio_get_debug_mode
      
      k = i
      do j = 1, len
         call set_buffer_char(sbr, io, k, s(j:j))
         k = k + 1
      enddo
      
      if (pio_get_debug_mode()) then
         k = k - 1
         write(PIO_DEBUG, *) 'In ', sbr, ': ',
     +        buffer_name(io), ' buffer from loc ', i,
     +        ' to loc ', k, ' is set to |', s(1:len), '|'
      endif
      
      return
      end
      
C     
C     Specialized routines for the scanning buffers.
C     
      
      integer function find_last_valid_char_index(sbr)
C     Find the index of the last valid character in buffer(PIO_IN).
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      
C     Local variables.
      character bad, c
      integer i
      
C     External functions.
      logical pio_get_debug_mode
      character int_to_char
      
C     Apparently, Fortran pads the input buffer with spaces.
      bad = int_to_char(32)
      
      do i = MAX_BUFFER_LOCATION - 1, MIN_BUFFER_LOCATION, -1
         c = buffer(PIO_IN)(i:i)
         if (c .ne. bad) then
            if (pio_get_debug_mode()) then
               write(PIO_DEBUG, *)'In ', sbr, ': ',
     +              'characters |', c, '| and |', bad, '| differ.'
            endif
            goto 10
         endif
      enddo
      
C     Getting here means there is no valid character in the buffer, ie
C     the line read is empty.
      i = 0
      
   10 continue
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) 'In ', sbr, ': ',
     +        'last valid char read index is ', i
      endif

      find_last_valid_char_index = i
      
      return
      end
      
      
      subroutine fill_scan_current_char(sbr)
C     Set scan_current_char to the current value in buffer(PIO_IN).
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      
C     Local variable.
      integer i
      
C     External function.
      logical pio_get_debug_mode
      
      i = buffer_ptr(PIO_IN)
      scan_current_char = buffer(PIO_IN)(i:i)
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) 'In ', sbr, ': ',
     +        'set current char to |', scan_current_char, '|'
      endif
      
      return
      end
      
      
      subroutine advance_and_fill_scan_current_char(sbr)
C     Set scan_current_char to the next character value in buffer(PIO_IN).
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input argument.
      character sbr*(*)
      
      call incr_buffer_ptr(sbr, PIO_IN)
      call fill_scan_current_char(sbr)
      
      return
      end
      
      
      subroutine fill_buffer(sbr, ic)
C     Read a line to fill buffer(PIO_IN),
C     set scan_current_char to the first character value in buffer(PIO_IN).
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer ic
      
C     Local variables.
      character c
      integer i, j, lim
      
C     External functions.
      logical pio_get_debug_mode
      character int_to_char
      integer find_last_valid_char_index
      
      call init_buffer(sbr, PIO_IN)
      
      i = MIN_BUFFER_LOCATION
      j = MAX_BUFFER_LOCATION - 1
      read(ic, '(a)', err=998, end=999) buffer(PIO_IN)(i:j)
      goto 10
      
  998 call error_read_error(sbr, ic)
  999 call error_end_of_file(sbr, ic)
      
   10 continue
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) 'In ', sbr, ': ',
     +        'contents of buffer is |',
     +        buffer(PIO_IN)(i:j), '|'
      endif
      
      lim = find_last_valid_char_index(sbr)
      
C     Put back the newline character '\n' at the end of the line read.
      c = int_to_char(10)
      lim = lim + 1
      buffer(PIO_IN)(lim:lim) = c
      buffer_lim(PIO_IN) = lim
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) 'In ', sbr, ': ',
     +        'number of valid char read is ', lim
         write(PIO_DEBUG, *) 'In ', sbr, ': ',
     +        'valid contents of buffer is |',
     +        buffer(PIO_IN)(1:lim), '|'
      endif
      
      call fill_scan_current_char(sbr)
      
      return
      end
      
      
      character function get_scan_current_char()
C     Get scan_current_char
      
      implicit none
      
      include 'buffers_definition.fh'
      
      get_scan_current_char = scan_current_char
      
      return
      end
      
      
      subroutine set_float_current_char(sbr, c)
C     Set the float buffer current char to c.
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input arguments.
      character sbr*(*), c
      
      if (floatbuf_ptr .gt. MAX_FLOAT_LENGTH) then
         call error_float_buffer_overflow(sbr)
      endif
      
      floatbuf(floatbuf_ptr:floatbuf_ptr) = c
      floatbuf_ptr = floatbuf_ptr + 1
      
      return
      end


      subroutine init_floatbuf
C     Initialize float buffer.
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Local variable
      integer i
      
      do i = 1, MAX_FLOAT_LENGTH
         floatbuf(i:i) = ' '
      enddo
      
      floatbuf_ptr = 1
      
      return
      end
      
      
      subroutine fill_floatbuf(sbr, c)
C     Fills the float buffer from the scanning buffer until the first
C     occurrence of character c (not included).
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input arguments.
      character sbr*(*), c
      
C     Local variable.
      character cur
      
C     External functions.
      logical end_of_buffer, pio_get_debug_mode
      character get_scan_current_char
      
      call init_floatbuf
      
   10 continue
      
      cur = get_scan_current_char()
      
      if (cur .ne. c) then
         
         call set_float_current_char(sbr, cur)
         
         if (end_of_buffer(PIO_IN)) then
            call error_char_not_found(sbr, c)
         endif
         
         call advance_and_fill_scan_current_char(sbr)
         
      else
         goto 20
      endif
      
      goto 10
      
   20 continue
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) 'In ', sbr, ': ',
     +        'contents of float buffer is |',
     +        floatbuf, '|'
      endif
      
      return
      end
      
      
      subroutine set_string_current_char(sbr, c)
C     Set the string buffer current char to c.
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input arguments.
      character sbr*(*), c
      
      if (stringbuf_ptr .gt. MAX_STRING_LENGTH) then
         call error_string_buffer_overflow(sbr)
      endif
      
      stringbuf(stringbuf_ptr:stringbuf_ptr) = c
      stringbuf_ptr = stringbuf_ptr + 1
      
      return
      end
      
      
      subroutine set_ident_current_char(sbr, c)
C     Set the ident buffer current char to c.
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input arguments.
      character sbr*(*), c
      
      if (identbuf_ptr .gt. PIO_MAX_IDENT_LENGTH) then
         call error_ident_buffer_overflow(sbr)
      endif
      
      identbuf(identbuf_ptr:identbuf_ptr) = c
      identbuf_ptr = identbuf_ptr + 1
      
      return
      end
      
      
      logical function is_decimal_digit(d)
C     Predicate to check if a character d is a decimal digit.
C     Used by the function scan_three_digits.
      
      implicit none
      
C     Input argument.
      integer d
      
      is_decimal_digit = (d .ge. ASCII0) .and. (d .le. ASCII9)
      
      return
      end
      
      
      integer function scan_three_digits(sbr)
C     Scan three decimal digits.
C     Used only by the function get_escaped_char.
      
      implicit none
      
C     Input argument.
      character sbr*(*)
      
C     Local variables.
      character c
      integer n, d, i
      
C     External functions.
      logical is_decimal_digit
      character get_scan_current_char
      integer char_to_int
      
      n = 0
      d = 0
      
      do i = 1, 3
         
         c = get_scan_current_char()
C     Some compilers may complain, but there is no precision loss here
C     since n should be a decimal digit.
         n = char_to_int(c)
         
         if (is_decimal_digit(n)) then
            
            d = 10 * d + n - ASCII0
            
            if (i .lt. 3) then
               call advance_and_fill_scan_current_char
     +              ('scan_three_digits')
            endif
            
         else
            call error_malformed_escaped_char(sbr, c)
         endif
         
      enddo
      
      scan_three_digits = d
      
      return
      end
      
      
      character function get_escaped_char(sbr)
C     Fill the string buffer from the scanning buffer with the current
C     escaped character.
C     Used only by the functions fill_stringbuf and fill_identbuf.
      
      implicit none
      
C     Input argument.
      character sbr*(*)
      
C     Local variables.
      character backslash, c
      integer d
      
C     External functions.
      character backslash_char, get_scan_current_char, int_to_char
      integer scan_three_digits
      
      backslash = backslash_char()
      
      call verify_scan_current_char_and_advance(sbr, backslash)
      
      c = get_scan_current_char()
      
      if ((c .eq. backslash) .or.
     +     (c .eq. '"') .or.
     +     (c .eq. '''') .or.
     +     (c .eq. ' ')) then
         continue
      else if (c .eq. 'b') then
         c = int_to_char(8)
      else if (c .eq. 'n') then
         c = int_to_char(10)
      else if (c .eq. 'r') then
         c = int_to_char(13)
      else if (c .eq. 't') then
         c = int_to_char(9)
      else
         d = scan_three_digits(sbr)
         c = int_to_char(d)
      endif
      
      get_escaped_char = c
      
      return
      end
      
      
      integer function fill_stringbuf(sbr)
C     Fill the string buffer from the scanning buffer until the first
C     occurrence of (non escaped) character '"' (not included).
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input argument.
      character sbr*(*)
      
C     Local variables.
      character escaping_char, stopping_char, cur
      integer size
      
C     External functions.
      logical end_of_buffer
      character backslash_char, get_scan_current_char, get_escaped_char
      
      escaping_char = backslash_char()
      stopping_char = '"'
      
      size = 0
      stringbuf_ptr = 1
      
   10 continue
      
      cur = get_scan_current_char()
      
      if (cur .ne. stopping_char) then
         
         if (cur .eq. escaping_char) then
            cur = get_escaped_char(sbr)
         endif
         
         call set_string_current_char(sbr, cur)
         size = size + 1
         
         if (end_of_buffer(PIO_IN)) then
            call error_char_not_found(sbr, stopping_char)
         endif
         
         call advance_and_fill_scan_current_char(sbr)
         
      else
         goto 20
      endif
      
      goto 10
      
   20 continue
      
      fill_stringbuf = size
      
      return
      end
      
      
      integer function fill_identbuf(sbr)
C     Fill the ident buffer from the scanning buffer until the first
C     occurrence of (non escaped) character '"' (not included).
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input argument.
      character sbr*(*)
      
C     Local variables.
      character escaping_char, stopping_char, cur
      integer size
      
C     External functions.
      logical end_of_buffer
      character backslash_char, get_scan_current_char, get_escaped_char
      
      escaping_char = backslash_char()
      stopping_char = '"'
      
      size = 0
      identbuf_ptr = 1
      
   10 continue
       
      cur = get_scan_current_char()
      
      if (cur .ne. stopping_char) then
         
         if (cur .eq. escaping_char) then
            cur = get_escaped_char(sbr)
         endif
         
         call set_ident_current_char(sbr, cur)
         size = size + 1
         
         if (end_of_buffer(PIO_IN)) then
            call error_char_not_found(sbr, stopping_char)
         endif
         
         call advance_and_fill_scan_current_char(sbr)
         
      else
         goto 20
      endif
      
      goto 10
      
   20 continue
      
      fill_identbuf = size
      
      return
      end
      
      
      character*MAX_STRING_LENGTH function get_stringbuf(sbr, len)
C     Get the len first bytes of the string buffer.
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer len
      
      if (len .ge. stringbuf_ptr) then
         call error_length_too_long(sbr, len, stringbuf_ptr - 1)
      endif
      
      get_stringbuf(1:len) = stringbuf(1:len)
      
      return
      end
      
      
      character*PIO_MAX_IDENT_LENGTH function get_identbuf(sbr, len)
C     Get the len first bytes of the ident buffer.
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer len
      
      if (len .ge. identbuf_ptr) then
         call error_length_too_long(sbr, len, identbuf_ptr - 1)
      endif
      
      get_identbuf(1:len) = identbuf(1:len)
      
      return
      end
      
C     
C     Specialized routines for the printing buffers.
C     
      
      subroutine string_of_int(sbr, n, len, si)
C     String of the integer n.
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer n
      
C     Output arguments.
      character si*MAX_INT_LENGTH
      integer len
      
C     Local variables.
      character c
      integer tmp, d, i, j
      
C     External function.
      character int_to_char
      
      intbuf_ptr = 1
      tmp = n
      
   10 continue
      
C     Some compilers may complain, but there is no precision loss here
C     since the result of mod is less than 10.
      d = mod(tmp, 10) + ASCII0
      c = int_to_char(d)
      
      intbuf(intbuf_ptr:intbuf_ptr) = c
      if (intbuf_ptr .lt. MAX_INT_LENGTH) then
         intbuf_ptr = intbuf_ptr + 1
      else
         call error_integer_buffer_overflow(sbr)
      endif
      
      tmp = tmp/10
      
      if (tmp .gt. 0) then
         goto 10
      endif
      
      j = 1
      len = intbuf_ptr - 1
      do i = len, 1, -1
         si(j:j) = intbuf(i:i)
         j = j + 1
      enddo
      
      return
      end
      
      
      subroutine string_of_float(f, len, sf)
C     String of the float f with format PRINT_FLOAT_FORMAT.
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input argument.
      FLOAT f
      
C     Output arguments.
      character sf*MAX_FLOAT_LENGTH
      integer len
      
      call init_floatbuf
      
      write(floatbuf, 1001) f
 1001 format (PRINT_FLOAT_FORMAT)
      
      sf = floatbuf
      len = PRINT_FLOAT_FORMAT_LENGTH
      
      return
      end
      
      
      subroutine flush_buffer(sbr, oc)
C     Flush the print buffer into output channel oc,
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer oc
      
C     Local variable.
      integer end_loc
      
      end_loc = buffer_lim(PIO_OUT) - 1
      write(oc, '(a)', err=998) buffer(PIO_OUT)(1:end_loc)

C     Using either the provided flush routine or the built-in flush
C     function provided by the Gnu Fortran compiler.
      call flush(oc)
      
      call init_buffer(sbr, PIO_OUT)
      return
      
  998 call error_write_error(sbr, oc)
      
      return
      end


      character function backslash_char()
C     A portable way to obtain the backslash character.
      
      implicit none
      
C     External function.
      character int_to_char
      
C     The backslash character has ASCII code 92.
      backslash_char = int_to_char(92)
      
      return
      end
